// ---------------------------------------------------------------------------
// matrix.hpp - Funktionsbibliothek zur Arbeit mit Vektoren und Matrizen
//
//               Copyright Tilman Kuepper 2009 - 2017.
//    Distributed under the Boost Software License, Version 1.0.
//        (See accompanying file LICENSE_1_0.txt or copy at
//               http://www.boost.org/LICENSE_1_0.txt)
//
// Letzte Bearbeitung: 1. August 2017
// ---------------------------------------------------------------------------

#ifndef MATRIX_HPP_Defined
#define MATRIX_HPP_Defined

#include "matrix.h"
#include <assert.h>
#include <stdexcept>
#include <ostream>
#include <sstream>
#include <string>
#include <vector>
#include <complex>

// Namespace fuer Vektoren und Matrizen ("HM" = "Hochschule Muenchen")
namespace HMMatrix
{
    // -----------------------------------------------------------------------
    // Typdefinitionen fuer komplexe Zahlen und Vektoren. Diese werden
    // zur Nullstellenberechnung von Polynomen benoetigt, vergl. Roots().
    // -----------------------------------------------------------------------
    typedef std::complex<double>  DComplex;
    typedef std::vector<DComplex> DComplexVector;
 
    // -----------------------------------------------------------------------
    // Definition der eigentlichen Vektorklasse. Die einzelnen Elemente
    // des Vektors sind in einem std::vector<double> abgespeichert.
    // -----------------------------------------------------------------------
    class Vector
    {
        std::vector<double> v_;  // Elemente des Vektors

        // -------------------------------------------------------------------
        // Hilfsklasse zur Belegung eines Vektors, siehe Vector::Elements():
        // vec.Elements() = 11.1, 22.2, 33.3, 44.4;
        // -------------------------------------------------------------------
        class Inserter
        {
            Vector& vec_;  // Referenz auf den zu belegenden Vektor
            size_t idx_;   // Index des naechsten zu schreibenden Elements

            // Kein Zuweisungsoperator definiert
            Inserter& operator= (const Inserter&);

        public:
            // Konstruktor: Es wird eine Referenz auf den zu
            // belegenden Vektor uebergeben.
            explicit Inserter(Vector& vec) : vec_(vec)
            {
                idx_ = 0;
            }

            // Das erste Element des Vektors wird gesetzt.
            Inserter& operator= (double value)
            {
                vec_(idx_++) = value;
                return *this;
            }

            // Ein weiteres Element des Vektors wird gesetzt.
            Inserter& operator, (double value)
            {
                vec_(idx_++) = value;
                return *this;
            }
        };

    public:
        // Konstruktor: Die Dimension des Vektors wird uebergeben.
        explicit Vector(size_t dim)
        {
            v_.resize(dim);
        }

        // Die Dimension des Vektors wird zurueckgegeben.
        size_t Dimension() const
        {
            return v_.size();
        }

        // Der komplette Vektor wird mit Werten belegt (siehe matrix.h
        // fuer eine Liste aller moeglichen Werte des Parameters "init").
        void Init(int init)
        {
            v_init(Dimension(), Data(), init);
        }

        // Alle Vektorelemente werden auf den angegebenen Wert gesetzt.
        void Fill(double value)
        {
            v_fill(Dimension(), Data(), value);
        }

        // Ein Zeiger auf das erste Element des Vektors wird zurueckgegeben
        // (nur lesende Version).
        const double* Data() const
        {
            return &v_[0];
        }

        // Ein Zeiger auf das erste Element des Vektors wird zurueckgegeben
        // (lesende und schreibende Version).
        double* Data()
        {
            return &v_[0];
        }

        // Zugriff auf ein einzelnes Element des Vektors
        // (nur lesende Version).
        double const& operator()(size_t idx) const
        {
            return v_.at(idx);
        }

        // Zugriff auf ein einzelnes Element des Vektors
        // (lesende und schreibende Version).
        double& operator()(size_t idx)
        {
            return v_.at(idx);
        }

        // Zugriff auf ein einzelnes Element des Vektors
        // (nur lesende Version).
        double const& operator[](size_t idx) const
        {
            return v_.at(idx);
        }

        // Zugriff auf ein einzelnes Element des Vektors
        // (lesende und schreibende Version).
        double& operator[](size_t idx)
        {
            return v_.at(idx);
        }

        // Hilfsfunktion zum Belegen aller Elemente des Vektors:
        // vec.Elements() = 11.1, 22.2, 33.3, 44.4;
        Inserter Elements()
        {
            Inserter ins(*this);
            return ins;
        }

        // Vektoren auf Gleichheit pruefen.
        bool operator== (const Vector& other) const
        {
            if(size() != other.size()) return false;  // Bug-Fix: 24.06.2017
            return MAT_OK == v_equal(Dimension(), Data(), other.Data());
        }

        // Vektoren auf Ungleichheit pruefen.
        bool operator!= (const Vector& other) const
        {
            if(size() != other.size()) return true;  // Bug-Fix: 24.06.2017
            return MAT_OK != v_equal(Dimension(), Data(), other.Data());
        }

        // Betrag des Vektors berechnen und zurueckgeben.
        double Abs() const
        {
            return v_abs(Dimension(), Data());
        }

        // Vektor normalisieren.
        void Normalize()
        {
            int ok = v_normalize(Dimension(), Data());
            if(MAT_OK != ok) throw std::runtime_error(mat_errmsg[ok]);
        }

        // Normalisierten Vektor berechnen und zurueckgeben.
        Vector Normal() const
        {
            Vector result(*this);
            int ok = v_normalize(result.Dimension(), result.Data());
            if(MAT_OK != ok) throw std::runtime_error(mat_errmsg[ok]);
            return result;
        }

        // Vektoren addieren.
        Vector& operator+= (const Vector& other)
        {
            assert(this->Dimension() == other.Dimension());
            if(this->Dimension() != other.Dimension())
                throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

            v_fadd(Dimension(), Data(), other.Data(), 1.0);
            return *this;
        }

        // Vektoren subtrahieren.
        Vector& operator-= (const Vector& other)
        {
            assert(this->Dimension() == other.Dimension());
            if(this->Dimension() != other.Dimension())
                throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

            v_fadd(Dimension(), Data(), other.Data(), -1.0);
            return *this;
        }

        // Vektor mit angegebenem Wert multiplizieren.
        Vector& operator*= (double value)
        {
            v_fmul(Dimension(), Data(), value);
            return *this;
        }

        // Vektor durch angegebenen Wert dividieren.
        Vector& operator/= (double value)
        {
            v_fmul(Dimension(), Data(), 1.0 / value);
            return *this;
        }

        // Vektoren addieren.
        Vector operator+ (const Vector& other) const
        {
            Vector result(*this);
            result += other;
            return result;
        }

        // Vektoren subtrahieren.
        Vector operator- (const Vector& other) const
        {
            Vector result(*this);
            result -= other;
            return result;
        }

        // Vektor mit angegebenem Wert multiplizieren.
        Vector operator* (double value) const
        {
            Vector result(*this);
            result *= value;
            return result;
        }

        // Skalarprodukt von zwei Vektoren berechnen.
        double operator* (const Vector& other) const
        {
            assert(this->Dimension() == other.Dimension());
            if(this->Dimension() != other.Dimension())
                throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

            double result = v_dot(Dimension(), Data(), other.Data());
            return result;
        }

        // Vektor durch angegebenen Wert dividieren.
        Vector operator/ (double value) const
        {
            Vector result(*this);
            result /= value;
            return result;
        }

        // Das maximale Element des Vektors "vec" wird zurueckgegeben.
        double GetMax() const
        {
            return v_get_max(Dimension(), Data());
        }

        // Das minimale Element des Vektors "vec" wird zurueckgegeben.
        double GetMin() const
        {
            return v_get_min(Dimension(), Data());
        }

        // Alle reellen und komplexen Nullstellen eines Polynoms ermitteln.
        // In den Elementen 0, 1, ..., n des Vektors sind die Koeffizienten
        // des Polynoms a[0] x^n + a[1] x^(n-1) + ... + a[n] anzugeben.
        DComplexVector Roots(
            double epsilon = MAT_EPSILON, size_t max_iter = MAT_MAX_ITER,
            DComplex wdk_init = DComplex(MAT_WDK_INIT)) const
        {
            HM_COMPLEX z;
            size_t dim = Dimension();
            const double *pdata = Data();

            // Zur Bedeutung der Parameter epsilon, max_iter, wdk_init
            // vergl. Dokumentation der C-Funktion droots_ex().
            if(dim < 2 || *pdata == 0)
                throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

            // Speicherplatz fuer komplexen Ergebnisvektor besorgen;
            // der folgende Typecast ist seit C++11 explizit erlaubt.
            DComplexVector roots(dim - 1);
            HM_COMPLEX *ptr = reinterpret_cast<HM_COMPLEX*>(&roots[0]);

            // Aufruf der C-Funktion zur Nullstellenberechnung.
            z.val[0] = wdk_init.real(), z.val[1] = wdk_init.imag();
            int ok = v_droots2(dim, pdata, ptr, epsilon, max_iter, z);
            if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
            return roots;
        }

        // Den Wert eines Polynoms an der Stelle x berechnen. In den
        // Elementen 0, 1, ..., n des Vektors sind die Koeffizienten
        // des Polynoms a[0] x^n + a[1] x^(n-1) + ... + a[n] anzugeben.
        double PolyVal(double x) const
        {
            return v_dpolyval(Dimension(), Data(), x);
        }

        // Die FFT (Fast Fourier Transform, schnelle Fouriertransformation)
        // des (reellen) Vektors wird berechnet. Achtung: Die Anzahl der
        // Vektorelemente muss eine Zweierpotenz sein, ansonsten wird die
        // Methode mit einem Ausnahmefehler des Typs "std::runtime_error"
        // abgebrochen.
        DComplexVector FFT() const
        {
            // Speicherplatz fuer komplexen Ergebnisvektor besorgen;
            // der folgende Typecast ist seit C++11 explizit erlaubt.
            DComplexVector result(Dimension());
            HM_COMPLEX *ptr = reinterpret_cast<HM_COMPLEX*>(&result[0]);

            int ok = v_dfft(Dimension(), Data(), ptr);
            if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
            return result;
        }

        // Iterator auf Beginn der Vektor-Elemente.
        typedef double* iterator;
        iterator begin()
        {
            return Data();
        }

        // Iterator auf Beginn der Vektor-Elemente (nur Lesezugriff!).
        typedef const double* const_iterator;
        const_iterator begin() const
        {
            return Data();
        }

        // Endpunkt fuer Iteration durch Vektor-Elemente.
        iterator end()
        {
            return Data() + Dimension();
        }

        // Endpunkt fuer Iteration durch Vektor-Elemente (nur Lesezugriff!).
        const_iterator end() const
        {
            return Data() + Dimension();
        }

        // Aktuelle Anzahl der Elemente im Vektor ermitteln.
        size_t size() const
        {
            return v_.size();
        }

        // Ist der Vektor leer?
        bool empty() const
        {
            return v_.empty();
        }

        // Ein neues Element am Ende des Vektors hinzufuegen.
        void push_back(double value)
        {
            v_.push_back(value);
        }

        // Das letzte Element des Vektors wird entfernt.
        void pop_back(void)
        {
            v_.pop_back();
        }

        // Alle Elemente des Vektors entfernen.
        void clear(void)
        {
            v_.clear();
        }

        // Ab dem Index "from" werden die naechsten "count" Elemente
        // zu einem neuen Vektor zusammengefuegt und zurueckgegeben,
        // optional kann auch eine Schrittweite "step" angegeben werden.
        // Diese Methode wird mit einem std::runtime_error abgebrochen,
        // falls die gesuchten Elemente nicht existieren, also zu
        // grosse Werte fuer "from" oder "count" angegeben werden.
        Vector Slice(size_t from, size_t count, size_t step = 1) const
        {
            Vector result(0);
            size_t my_size = v_.size();
            result.v_.reserve(count);
            for(size_t pos = from; result.size() < count; pos += step)
            {
                if(pos >= my_size)
                    throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);
                result.push_back(v_[pos]);
            }
            return result;
        }
    };

    // -----------------------------------------------------------------------
    // Skalar mit Vektor multiplizieren (erster Faktor ist ein Skalar).
    // -----------------------------------------------------------------------
    Vector operator* (double factor, Vector v)
    {
        v *= factor;  // "v" wurde als Kopie uebergeben!
        return v;
    }

    // -----------------------------------------------------------------------
    // Kreuzprodukt von zwei dreidimensionalen Vektoren berechen.
    // -----------------------------------------------------------------------
    Vector CrossProduct(const Vector& v1, const Vector& v2)
    {
        assert(v1.Dimension() == 3);
        assert(v2.Dimension() == 3);
        if(v1.Dimension() != 3 || v2.Dimension() != 3)
            throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

        Vector result(3);
        v_cross(result.Data(), v1.Data(), v2.Data());
        return result;
    }

    // -----------------------------------------------------------------------
    // Die FFT (Fast Fourier Transform, schnelle Fouriertransformation) des
    // reellen Vektors "f" wird berechnet. Achtung: Die Anzahl der Elemente
    // des Vektors muss eine Zweierpotenz sein, ansonsten wird diese Funktion
    // mit einem Ausnahmefehler des Typs "std::runtime_error" abgebrochen.
    // -----------------------------------------------------------------------
    DComplexVector FFT(const Vector& f)
    {
        return f.FFT();
    }
        
    // -----------------------------------------------------------------------
    // Die FFT (Fast Fourier Transform, schnelle Fouriertransformation) des
    // komplexen Vektors "f" wird berechnet. Achtung: Die Anzahl der Elemente
    // des Vektors muss eine Zweierpotenz sein, ansonsten wird diese Funktion
    // mit einem Ausnahmefehler des Typs "std::runtime_error" abgebrochen.
    // -----------------------------------------------------------------------
    DComplexVector FFT(const DComplexVector& f)
    {
        // Speicherplatz fuer komplexen Ergebnisvektor besorgen;
        // der folgende Typecast ist seit C++11 explizit erlaubt.
        assert(f.size() != 0);
        DComplexVector result(f.size());
        HM_COMPLEX *ptr = reinterpret_cast<HM_COMPLEX*>(result.data());

        DComplexVector::const_iterator f_it = f.begin(), f_end = f.end();
        DComplexVector::iterator res_it = result.begin();
        while(f_it != f_end) *res_it++ = *f_it++;

        int ok = v_cfft(f.size(), ptr);
        if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
        return result;
    }

    // -----------------------------------------------------------------------
    // Die IFFT (Inverse Fourier Transform) des komplexen Vektors "y" wird
    // berechnet. Achtung: Die Anzahl der Elemente des Vektors muss eine
    // Zweierpotenz sein, sonst wird diese Funktion mit einem Ausnahmefehler
    // des Typs "std::runtime_error" abgebrochen.
    // -----------------------------------------------------------------------
    DComplexVector IFFT(const DComplexVector& y)
    {
        // Speicherplatz fuer komplexen Ergebnisvektor besorgen;
        // der folgende Typecast ist seit C++11 explizit erlaubt.
        assert(y.size() != 0);
        DComplexVector result(y.size());
        HM_COMPLEX *ptr = reinterpret_cast<HM_COMPLEX*>(result.data());

        DComplexVector::const_iterator y_it = y.begin(), y_end = y.end();
        DComplexVector::iterator res_it = result.begin();
        while(y_it != y_end) *res_it++ = *y_it++;

        int ok = v_ifft(y.size(), ptr);
        if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
        return result;
    }

    // -----------------------------------------------------------------------
    // Funktionen zum Einlesen von Vektoren in MATLAB-Schreibweise.
    // -----------------------------------------------------------------------

    // Vektor in MATLAB-Schreibweise aus std::basic_string<> einlesen
    template <class E, class T> Vector MATLABVector(
        const std::basic_string<E, T> &str)
    {
        enum { INIT, ELEMENT, SEPARATOR, DONE } state = INIT;
        const char* errmsg = mat_errmsg[MAT_ERR_FORMAT];
        std::basic_istringstream<E, T> strm(str);
        E character = 0;
        double value;

        // Ergebnisvektor wird waehrend des Einlesens mit Werten gefuellt
        Vector result(0);

        do
        {
            switch(state)
            {
            case INIT:  // Vektor muss mit '[' beginnen
                strm >> character;
                if(strm.good() == false) throw std::runtime_error(errmsg);
                if(character !=  E('[')) throw std::runtime_error(errmsg);
                state = ELEMENT;
                break;

            case ELEMENT:  // Ein Vektorelement einlesen
                strm >> value;
                if(strm.good() == false) throw std::runtime_error(errmsg);
                result.push_back(value); state = SEPARATOR;
                break;

            case SEPARATOR:  // Ein (optionales) Trennzeichen einlesen
                strm >> character;
                if(strm.good() == false) throw std::runtime_error(errmsg);
                if(character ==  E(',')) { state = ELEMENT; break; }
                if(character ==  E(';')) { state = ELEMENT; break; }
                if(character ==  E(']')) { state =    DONE; break; }
                strm.putback(E(character)); state = ELEMENT;
                break;

            default:  // Interner Fehler, ungueltiger Status
                assert(!"MATLABVector(), invalid state");
                throw std::runtime_error(errmsg);
            }
        }
        while(state != DONE);
        return result;
    }

    // Vektor in MATLAB-Schreibweise aus std::string bzw. std::wstring einlesen
    Vector MATLABVector(const std::string  &s) { return MATLABVector<>(s); }
    Vector MATLABVector(const std::wstring &s) { return MATLABVector<>(s); }

    // -----------------------------------------------------------------------
    // Stream-Manipulator zur Einstellung des Matrix-/Vektor-Ausgabeformats:
    // std::cout << MatrixFormat(6, 2, true, false, "; ") << myMatrix;
    //
    // Literaturtipp zur Implementierung von Stream-Manipulatoren:
    // Angelika Langer, Klaus Kreft, "Standard C++ IOStreams and Locales"
    // (Kapitel 3.2.2, "Manipulators with Parameters" und Kapitel 3.3.1,
    // "Using Stream Storage for Private Use: iword, pword and xalloc")
    // -----------------------------------------------------------------------
    class MatrixFormat
    {
        size_t width_, prec_;       // Feldbreite, Anzahl Nachkommastellen
        bool matNewln_, vecNewln_;  // Zeilenumbrueche gewuenscht?
        std::string separator_;     // Trennzeichen zwischen Elementen

    public:
        // Konstruktor: Feldbreite, Nachkommastellen, Zeilenumbrueche,
        // Trennzeichen zwischen einzelnen Elementen werden festgelegt.
        MatrixFormat(
            size_t width  = 8, size_t prec   = 2,
            bool matNewln = true, bool vecNewln = false,
            const std::string& separator = "; ") : separator_(separator)
        {
            width_     = width;
            prec_      = prec;
            matNewln_  = matNewln;
            vecNewln_  = vecNewln; 
        }

        // Zugriff auf einzelne Attribute von "MatrixFormat".
        size_t GetWidth () const { return width_;    }
        size_t GetPrec  () const { return prec_;     }
        bool GetMatNewln() const { return matNewln_; }
        bool GetVecNewln() const { return vecNewln_; }
        const char* GetSeparator() const { return separator_.c_str(); }

        // Zwei Instanzen von "MatrixFormat" werden getauscht.
        static void Swap(MatrixFormat& f1, MatrixFormat& f2)
        {
            std::swap(f1.width_    , f2.width_    );
            std::swap(f1.prec_     , f2.prec_     );
            std::swap(f1.matNewln_ , f2.matNewln_ );
            std::swap(f1.vecNewln_ , f2.vecNewln_ );
            std::swap(f1.separator_, f2.separator_);
        }

        // Eine Instanz von "MatrixFormat" kann einem Ausgabe-Stream
        // zugeordnet werden, um das Ausgabeformat von Matrizen und
        // Vektoren einzustellen. Dazu wird der "pword-Mechanismus" aus
        // der C++-Standard-Library benutzt. Die Funktion "GetPwordIdx"
        // gibt den Index zurueck, an dem die Formatierungsdaten im
        // Ausgabe-Stream abgelegt sind.
        static int GetPwordIdx()
        {
            static const int idx = std::ios_base::xalloc();
            return idx;
        }

        // Wenn ein Ausgabestream kopiert oder geloescht wird, muss auch
        // die dem Stream zugeordnete Instanz von "MatrixFormat" kopiert
        // oder geloescht werden. Dazu dient diese Callback-Funktion.
        static void Callback(std::ios_base::event ev, std::ios_base& os, int)
        {
            void*& pword = os.pword(GetPwordIdx());
            MatrixFormat*& pFormat = reinterpret_cast<MatrixFormat*&>(pword);
            if(ev == std::ios_base::erase_event)
            {
                delete pFormat;  // "MatrixFormat" loeschen
            }
            else if(ev == std::ios_base::copyfmt_event && pFormat)
            {
                try
                {
                    pFormat = new MatrixFormat(*pFormat);  // kopieren
                }
                catch(...)
                {
                    // Achtung: Es duerfen keine Ausnahmefehler aus der
                    // Callback-Funktion nach aussen gelangen!
                    pFormat = 0;
                    assert(!"copyfmt failed");
                }
            }
        }
    };

    // -----------------------------------------------------------------------
    // Stream-Operator zur Einstellung des Matrix-/Vektor-Ausgabeformats:
    // std::cout << MatrixFormat(6, 2, true, false, "; ") << myMatrix;
    // -----------------------------------------------------------------------
    template<class E, class T> std::basic_ostream<E, T>& operator<< (
        std::basic_ostream<E, T>& os, MatrixFormat fmt)
    {
        void*& pword = os.pword(MatrixFormat::GetPwordIdx());
        if(MatrixFormat*& pFormat = reinterpret_cast<MatrixFormat*&>(pword))
        {
            // Dem Stream war bereits ein "MatrixFormat" zugeordnet
            MatrixFormat::Swap(*pFormat, fmt);
        }
        else
        {
            // Instanz von "MatrixFormat" erzeugen und im Stream speichern
            os.register_callback(MatrixFormat::Callback, 0);
            pFormat = new MatrixFormat(fmt);
        }
        return os;
    }

    // -----------------------------------------------------------------------
    // Alle Elemente des Vektors "vec" auf Ausgabe-Stream "os" schreiben.
    // -----------------------------------------------------------------------
    template<class E, class T> std::basic_ostream<E, T>& operator<< (
        std::basic_ostream<E, T>& os, const Vector& vec)
    {
        // Wurde ein spezielles Ausgabeformat eingestellt?
        MatrixFormat fmt;  // Default-Format
        void*& pword = os.pword(MatrixFormat::GetPwordIdx());
        if(MatrixFormat*& pFormat = reinterpret_cast<MatrixFormat*&>(pword))
            fmt = *pFormat;  // spezielles Format
 
        std::streamsize oldp = os.precision();
        std::ios_base::fmtflags oldfmt = os.flags();
        os.flags(std::ios_base::fixed | std::ios_base::right);

        size_t dim = vec.Dimension();
        for(size_t idx = 0; idx < dim; ++idx)
        {
            if(idx) os << fmt.GetSeparator();
            os.width(fmt.GetWidth()); os.precision(fmt.GetPrec());
            os << vec(idx);
        }
        if(fmt.GetVecNewln()) os << "\n";
        os.flags(oldfmt); os.precision(oldp);
        return os;
    }

    // -----------------------------------------------------------------------
    // Definition der eigentlichen Matrixklasse. Die einzelnen Elemente
    // der Matrix sind in einem std::vector<double> abgespeichert.
    // -----------------------------------------------------------------------
    class Matrix
    {
        size_t dim_;             // Dimension der Matrix
        std::vector<double> m_;  // Elemente der Matrix

        // -------------------------------------------------------------------
        // Hilfsklasse zur Belegung einer Matrixzeile, siehe Matrix::Row():
        // mat.Row(1) = 11.1, 22.2, 33.3, 44.4;
        // -------------------------------------------------------------------
        class RowInserter
        {
            Matrix& mat_;       // Referenz auf die zu belegende Matrix
            size_t row_, col_;  // Pos. des naechsten zu schreibenden Elements

            // Kein Zuweisungsoperator
            RowInserter& operator= (const RowInserter&);

        public:
            // Konstruktor: Es werden eine Referenz auf die zu belegende
            // Matrix und die betreffende Zeilennummer uebergeben.
            RowInserter(Matrix& mat, size_t row) : mat_(mat), row_(row)
            {
                col_ = 0;
            }

            // Das erste Element der Matrixzeile wird gesetzt.
            RowInserter& operator= (double value)
            {
                mat_(row_, col_++) = value;
                return *this;
            }

            // Ein weiteres Element der Matrixzeile wird gesetzt.
            RowInserter& operator, (double value)
            {
                mat_(row_, col_++) = value;
                return *this;
            }
        };

    public:
        // Konstruktor: Die Dimension der Matrix wird uebergeben.
        explicit Matrix(size_t dim) : dim_(dim)
        {
            m_.resize(dim_ * dim_);
        }

        // Die Dimension der Matrix wird zurueckgegeben.
        size_t Dimension() const
        {
            return dim_;
        }

        // Die komplette Matrix wird mit Werten belegt (siehe "matrix.h"
        // fuer eine Liste aller moeglichen Werte des Parameters "init").
        void Init(int init)
        {
            m_init(dim_, Data(), init);
        }

        // Alle Matrixelemente werden auf den angegebenen Wert gesetzt.
        void Fill(double value)
        {
            m_fill(dim_, Data(), value);
        }

        // Ein Zeiger auf das erste Element der Matrix wird zurueckgegeben
        // (nur lesende Version).
        const double* Data() const
        {
            return &m_[0];
        }

        // Ein Zeiger auf das erste Element der Matrix wird zurueckgegeben
        // (lesende und schreibende Version).
        double* Data()
        {
            return &m_[0];
        }

        // Zugriff auf ein einzelnes Element der Matrix
        // (nur lesende Version).
        double const& operator()(size_t row, size_t col) const
        {
            return m_.at(row * dim_ + col);
        }

        // Zugriff auf ein einzelnes Element der Matrix
        // (lesende und schreibende Version).
        double& operator()(size_t row, size_t col)
        {
            return m_.at(row * dim_ + col);
        }

        // Matrizen addieren.
        Matrix& operator+= (const Matrix& other)
        {
            assert(this->Dimension() == other.Dimension());
            if(this->Dimension() != other.Dimension())
                throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

            m_fadd(dim_, Data(), other.Data(), 1.0);
            return *this;
        }

        // Matrizen subtrahieren.
        Matrix& operator-= (const Matrix& other)
        {
            assert(this->Dimension() == other.Dimension());
            if(this->Dimension() != other.Dimension())
                throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

            m_fadd(dim_, Data(), other.Data(), -1.0);
            return *this;
        }

        // Matrizen multiplizieren.
        Matrix& operator*= (const Matrix& other)
        {
            Matrix tmp = (*this) * other;
            std::swap(m_, tmp.m_);
            return *this;
        }

        // Matrix mit angegebenem Wert multiplizieren.
        Matrix& operator*= (double value)
        {
            m_fmul(dim_, Data(), value);
            return *this;
        }

        // Matrix durch angegebenen Wert dividieren.
        Matrix& operator/= (double value)
        {
            m_fmul(dim_, Data(), 1.0 / value);
            return *this;
        }

        // Matrizen addieren.
        Matrix operator+ (const Matrix& other) const
        {
            Matrix result(*this);
            result += other;
            return result;
        }

        // Matrizen subtrahieren.
        Matrix operator- (const Matrix& other) const
        {
            Matrix result(*this);
            result -= other;
            return result;
        }

        // Matrizen multiplizieren.
        Matrix operator* (const Matrix& other) const
        {
            if(dim_ != other.dim_)
            {
                assert(!"matrix mult., dimensions don't match.");
                throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);
            }

            Matrix result(dim_);
            m_mul(dim_, result.m_.data(), m_.data(), other.m_.data());
            return result;
        }

        // Matrix mit angegebenem Wert multiplizieren.
        Matrix operator* (double value) const
        {
            Matrix result(*this);
            result *= value;
            return result;
        }

        // Matrix und Vektor multiplizierern.
        Vector operator* (const Vector& other) const
        {
            assert(this->Dimension() == other.Dimension());
            if(this->Dimension() != other.Dimension())
                throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

            Vector result(dim_);
            m_vmul(dim_, result.Data(), Data(), other.Data());
            return result;
        }

        // Matrix durch angegebenen Wert dividieren.
        Matrix operator/ (double value) const
        {
            Matrix result(*this);
            result /= value;
            return result;
        }

        // Matrizen auf Gleichheit pruefen.
        bool operator== (const Matrix& other) const
        {
            if(size() != other.size()) return false;  // Bug-Fix: 24.06.2017
            return MAT_OK == m_equal(dim_, Data(), other.Data());
        }

        // Matrizen auf Ungleichheit pruefen.
        bool operator!= (const Matrix& other) const
        {
            if(size() != other.size()) return true;  // Bug-Fix: 24.06.2017
            return MAT_OK != m_equal(dim_, Data(), other.Data());
        }

        // Matrix invertieren.
        void Invert()
        {
            int ok = m_invert(dim_, Data());
            if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
        }

        // Inverse der Matrix berechnen und zurueckgeben.
        Matrix Inverse() const
        {
            Matrix result(*this);
            result.Invert();
            return result;
        }

        // Matrix transponieren.
        void Transpose()
        {
            m_transpose(dim_, Data());
        }

        // Transponierte der Matrix berechnen und zurueckgeben.
        Matrix GetTranspose() const
        {
            Matrix result(*this);
            result.Transpose();
            return result;
        }

        // Determinante der Matrix berechnen und zurueckgeben.
        double Determinant() const
        {
            int ok;
            double result = m_det(dim_, Data(), &ok);
            if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
            return result;
        }

        // Ist die Matrix symmetrisch (Rueckgabe = true) oder ist sie nicht
        // symmetrisch (Rueckgabe = false)? Bei einer symmetrischen Matrix
        // weichen die an der Hauptdiagonalen gespiegelten Matrixelemente
        // maximal um den Betrag "epsilon" voneinander ab.
        bool IsSymmetric(double epsilon = 0) const
        {
            return m_is_symmetric(dim_, Data(), epsilon) == MAT_OK;
        }

        // Die Matrix wird symmetrisch gemacht. Entweder dadurch, dass die
        // Elemente oberhalb der Hauptdiagonalen nach unten kopiert werden
        // (mode = MAT_USE_UPPER) oder dadurch, dass die Elemente unterhalb
        // der Hauptdiagonalen nach oben kopiert werden (mode = MAT_USE_LOWER)
        // oder dadurch, dass die Mittelwerte aller an der Hauptdiagonalen
        // gespiegelten Elemente berechnet werden (mode = MAT_USE_MEAN).
        void MakeSymmetric(int mode = MAT_USE_MEAN)
        {
            m_make_symmetric(dim_, Data(), mode);
        }

        // QR-Zerlegung der Matrix durchfuehren.
        void QRDecomp(Matrix& q, Matrix& r) const
        {
            assert(q.Dimension() == dim_);
            assert(r.Dimension() == dim_);
            if(q.Dimension() != dim_ || r.Dimension() != dim_)
                throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

            int ok = m_qr_decomp(dim_, Data(), q.Data(), r.Data());
            if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
        }

        // LU-Zerlegung der Matrix durchfuehren. Im Laufe der Berechnung
        // werden geeignete Zeilenvertauschungen durchgefuehrt, die in der
        // Permutationsmatrix P wiedergegeben sind (P * MAT = L * U). Der
        // Rueckgabewert der Methode betraegt (-1)^n, wobei n die Anzahl
        // der Zeilenvertauschungen ist.
        double LUDecomp(Matrix& l, Matrix& u, Matrix& p) const
        {
            assert(l.Dimension() == dim_);
            assert(u.Dimension() == dim_);
            assert(p.Dimension() == dim_);
            if(l.dim_ != dim_ || u.dim_ != dim_ || p.dim_ != dim_)
                throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

            double sgn;
            int ok = m_lu_decomp(dim_, Data(), u.Data(), p.Data(), &sgn);
            if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);

            // Die beiden Dreiecksmatrizen L, U wurden von der C-Funktion
            // in einer "kombinierten Matrix" zurueckgegeben. Sie werden
            // nun in zwei separate Matrizen aufgeteilt.
            l.Init(MAT_INIT_IDENTITY);
            for(size_t row = 0; row < dim_; ++row)
                for(size_t col = 0; col < row; ++col)
                    std::swap(l(row, col), u(row, col));
            return sgn;
        }

        // Die Koeffizienten des charakteristischen Polynoms der
        // Matrix werden ermittelt und als Vektor zurueckgegeben.
        Vector CharPol() const
        {
            Vector pol(dim_ + 1);
            int ok = m_charpol(dim_, Data(), pol.Data());
            if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
            return pol;
        }

        // Mittels zyklischem Jacobi-Verfahren werden alle Eigenwerte und
        // -vektoren einer symmetrischen (!) Matrix ermittelt. Die
        // Eigenwerte werden auf der Hauptdiagonalen der Matrix "eig",
        // die Eigenvektoren in den jeweiligen Spalten der Matrix "vec"
        // abgelegt. Die Eigenwerte sind nicht nach ihrer  Groesse sortiert.
        // Achtung: Bei nicht-symmetrischen Matrizen liefert die Methode
        // "Jacobi" keine korrekten Ergebnisse (Eigenwerte bzw. -vektoren)!
        void Jacobi(Matrix& eig, Matrix& vec, double epsilon = MAT_EPSILON,
            size_t max_iter = MAT_MAX_ITER) const
        {
            assert(eig.Dimension() == dim_);
            assert(vec.Dimension() == dim_);
            if(eig.Dimension() != dim_ || vec.Dimension() != dim_)
                throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

            int ok = m_jacobi2(dim_, Data(), eig.Data(), vec.Data(),
                epsilon, max_iter);
            if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
        }

        // Der betragsmaessig groesste Eigenwert und der dazu passende
        // Eigenvektor der aktuellen Matrix werden berechnet und ueber
        // den Referenzparameter "eig" (Eigenwert) sowie als Rueckgabewert
        // (Eigenvektor) zurueckgegeben. Die Berechnung des Eigenwerts bzw.
        // des Eigenvektors erfolgt ueber eine von-Mises-Vektoriteration.
        // Achtung: Insbesondere bei doppelten, eng benachbarten oder
        // komplexen Eigenwerten konvergiert dieses Verfahren nur sehr
        // langsam oder ueberhaupt nicht. In solchen Faellen sollten die
        // Parameter "max_iter" und "epsilon" auf geeignete Werte gesetzt
        // werden, um ein Einfrieren des Programmablaufs zu vermeiden.
        Vector Mises(double& eig, double epsilon = MAT_EPSILON,
            size_t max_iter = MAT_MAX_ITER) const
        {
            Vector vec(dim_);
            int ok = m_mises2(dim_, Data(), &eig, vec.Data(),
                max_iter, epsilon);
            if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
            return vec;
        }

        // Zwei Zeilen der Matrix tauschen.
        void SwapRow(size_t row1, size_t row2)
        {
            m_swap_row(dim_, Data(), row1, row2);
        }

        // Einen einzelnen Zeilenvektor zurueckgeben.
        Vector GetRow(size_t row) const
        {
            Vector result(dim_);
            m_get_row(dim_, row, Data(), result.Data());
            return result;
        }

        // Einen einzelnen Spaltenvektor zurueckgeben.
        Vector GetCol(size_t col) const
        {
            Vector result(dim_);
            m_get_col(dim_, col, Data(), result.Data());
            return result;
        }

        // Die Hauptdiagonalelemente in Form eines Vektors zurueckgeben.
        Vector GetDiag() const
        {
            Vector result(dim_);
            m_get_diag(dim_, Data(), result.Data());
            return result;
        }

        // Das maximale Element der Matrix wird zurueckgegeben.
        double GetMax() const
        {
            return m_get_max(dim_, Data());
        }

        // Das minimale Element der Matrix wird zurueckgegeben.
        double GetMin() const
        {
            return m_get_min(dim_, Data());
        }

        // Hilfsfunktion zur Belegung einer kompletten Matrixzeile:
        // mat.Row(1) = 11.1, 22.2, 33.3, 44.4;
        RowInserter Row(size_t row)
        {
            RowInserter ins(*this, row);
            return ins;
        }

        // Iterator auf Beginn der Matrix-Elemente.
        typedef double* iterator;
        iterator begin()
        {
            return Data();
        }

        // Iterator auf Beginn der Matrix-Elemente (nur Lesezugriff!).
        typedef const double* const_iterator;
        const_iterator begin() const
        {
            return Data();
        }

        // Endpunkt fuer Iteration durch Matrix-Elemente.
        iterator end()
        {
            return Data() + dim_ * dim_;
        }

        // Endpunkt fuer Iteration durch Matrix-Elemente (nur Lesezugriff!).
        const_iterator end() const
        {
            return Data() + dim_ * dim_;
        }

        // Gesamte Anzahl der Elemente in der Matrix ermitteln.
        size_t size() const
        {
            return m_.size();
        }
    };

    // -----------------------------------------------------------------------
    // Skalar mit Matrix multiplizieren (erster Faktor ist ein Skalar).
    // -----------------------------------------------------------------------
    Matrix operator* (double factor, Matrix m)
    {
        m *= factor;  // "m" wurde als Kopie uebergeben!
        return m;
    }

    // -----------------------------------------------------------------------
    // Alle Elemente der Matrix "mat" auf Ausgabe-Stream "os" schreiben.
    // -----------------------------------------------------------------------
    template<class E, class T> std::basic_ostream<E, T>& operator<< (
        std::basic_ostream<E, T>& os, const Matrix& mat)
    {
        // Wurde ein spezielles Ausgabeformat eingestellt?
        MatrixFormat fmt;  // Default-Format
        void*& pword = os.pword(MatrixFormat::GetPwordIdx());
        if(MatrixFormat*& pFormat = reinterpret_cast<MatrixFormat*&>(pword))
            fmt = *pFormat;  // spezielles Format

        std::streamsize oldp = os.precision();
        std::ios_base::fmtflags oldfmt = os.flags();
        os.flags(std::ios_base::fixed | std::ios_base::right);

        size_t dim = mat.Dimension();
        for(size_t row = 0; row < dim; ++row)
        {
            for(size_t col = 0; col < dim; ++col)
            {
                if(col) os << fmt.GetSeparator();
                os.width(fmt.GetWidth()); os.precision(fmt.GetPrec());
                os << mat(row, col);
            }
            if(fmt.GetMatNewln())  os << "\n";
            else if(row < dim - 1) os << fmt.GetSeparator();
        }
        os.flags(oldfmt); os.precision(oldp);
        return os;
    }

    // -----------------------------------------------------------------------
    // Funktionen zum Loesen von linearen Gleichungssystemen
    // -----------------------------------------------------------------------

    // -----------------------------------------------------------------------
    // Das lineare Gleichungssystem A * x = y wird geloest. Dazu wird zuerst
    // eine QR-Zerlegung der Matrix durchgefuehrt und danach die Loesung des
    // Gleichungssystems durch Rueckwaerts-Einsetzen berechnet. Falls keine
    // Loesung bestimmt werden kann (Matrix singulaer oder zu wenig Speicher)
    // wird ein Ausnahmefehler des Typs "std::runtime_error" ausgeloest.
    // -----------------------------------------------------------------------
    Vector LinSolve(const Matrix& a, const Vector& y)
    {
        assert(a.Dimension() == y.Dimension());
        if(a.Dimension() != y.Dimension())
            throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

        size_t dim = a.Dimension();
        Vector x(dim);
        int ok = lin_solve(dim, x.Data(), a.Data(), y.Data());
        if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
        return x;
    }

    // -----------------------------------------------------------------------
    // Das lineare Gleichungssystem (QR) * x = y wird geloest. Vor dem Aufruf
    // von "LinSolveQR" muss zunaechst eine QR-Zerlegung der Matrix erfolgen,
    // zum Beispiel mit "Matrix::QRDecomp()". (Alternativ kann die Funktion
    // "LinSolve" aufgerufen werden, wo beide Schritte - QR-Zerlegung und die
    // Loesung des Gleichungssystems - gemeinsam durchgefuehrt werden.) Falls
    // keine Loesung bestimmt werden kann (Matrix singulaer, zu wenig freier
    // Speicher) wird eine Ausnahme des Typs "std::runtime_error" ausgeloest.
    // -----------------------------------------------------------------------
    Vector LinSolveQR(const Matrix& q, const Matrix& r, const Vector& y)
    {
        assert(q.Dimension() == r.Dimension());
        assert(q.Dimension() == y.Dimension());
        if(q.Dimension() != r.Dimension() || q.Dimension() != y.Dimension())
            throw std::runtime_error(mat_errmsg[MAT_ERR_DIM]);

        size_t dim = q.Dimension();
        Vector x(dim);
        int ok = lin_solve_qr(dim, x.Data(), q.Data(), r.Data(), y.Data());
        if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
        return x;
    }

    // -----------------------------------------------------------------------
    // Funktionen zum Speichern und Laden von Matrizen im CSV-Format.
    // -----------------------------------------------------------------------

    // Speichern, Variante 1: Dateiname wird uebergeben
    void WriteCSV(const Matrix& mat, const char* fname)
    {
        int ok = m_writecsv(mat.Dimension(), mat.Data(), fname);
        if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
    }

    // Speichern, Variante 2: File-Handle wird uebergeben
    void WriteCSV(const Matrix& mat, FILE* file)
    {
        int ok = m_writecsv2(mat.Dimension(), mat.Data(), file);
        if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
    }

    // Laden, Variante 1: Dateiname wird uebergeben
    void ReadCSV(Matrix& mat, const char* fname)
    {
        int ok = m_readcsv(mat.Dimension(), mat.Data(), fname);
        if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
    }

    // Laden, Variante 2: File-Handle wird uebergeben
    void ReadCSV(Matrix& mat, FILE* file)
    {
        int ok = m_readcsv2(mat.Dimension(), mat.Data(), file);
        if(ok != MAT_OK) throw std::runtime_error(mat_errmsg[ok]);
    }

    // -----------------------------------------------------------------------
    // Funktionen zum Einlesen von (quadratischen) Matrizen im MATLAB-Format.
    // -----------------------------------------------------------------------

    // Quadratische MATLAB-Matrix aus std::basic_string<> einlesen
    template <class E, class T> Matrix MATLABMatrix(
        const std::basic_string<E, T> &str)
    {
        enum 
        {
            INIT, ELEMENT_1ST_LINE, SEPARATOR_1ST_LINE, ELEMENT,
            SEPARATOR, SEMICOLON, END_OF_MATRIX, DONE
        } state = INIT;

        const char* errmsg = mat_errmsg[MAT_ERR_FORMAT];
        std::basic_istringstream<E, T> strm(str);
        size_t dim = 0, row = 0, col = 0;
        E character = 0;
        double value;

        // "Zwischenspeicher" fuer Matrixelemente waehrend des Einlesevorgangs
        std::vector<double> elements;

        do
        {
            switch(state)
            {
            case INIT:  // Matrix muss mit '[' beginnen
                strm >> character;
                if(!strm.good()) throw std::runtime_error(errmsg);
                if(character != E('[')) throw std::runtime_error(errmsg);
                state = ELEMENT_1ST_LINE;
                break;

            case ELEMENT_1ST_LINE:  // Element in der 1. Zeile einlesen
                strm >> value;
                if(!strm.good()) throw std::runtime_error(errmsg);
                elements.push_back(value);
                ++dim, state = SEPARATOR_1ST_LINE;
                break;

            case SEPARATOR_1ST_LINE:  // Trennzeichen in der 1. Zeile einlesen
                strm >> character;
                if(!strm.good()) throw std::runtime_error(errmsg);
                if(character == E(']') && dim == 1) state = DONE;
                else if(character == E(',')) state = ELEMENT_1ST_LINE;
                else if(character == E(';')) ++row, col = 0, state = ELEMENT;
                else strm.putback(E(character)), state = ELEMENT_1ST_LINE;
                if(row >= dim) throw std::runtime_error(errmsg);
                break;

            case ELEMENT:  // Ein weiteres Matrixelement einlesen
                strm >> value;
                if(!strm.good()) throw std::runtime_error(errmsg);
                elements.push_back(value);
                if(++col < dim) state = SEPARATOR;
                else if(++row < dim) col = 0, state = SEMICOLON;
                else state = END_OF_MATRIX;
                break;

            case SEPARATOR:  // Ein weiteres Trennzeichen einlesen
                strm >> character;
                if(!strm.good()) throw std::runtime_error(errmsg);
                if(character != E(',')) strm.putback(E(character));
                state = ELEMENT;
                break;

            case SEMICOLON:  // Am Zeilenende wird ein Semikolon erwartet
                strm >> character;
                if(!strm.good()) throw std::runtime_error(errmsg);
                if(character != E(';')) throw std::runtime_error(errmsg);
                state = ELEMENT;
                break;

            case END_OF_MATRIX:  // Matrix muss mit ']' beendet werden
                strm >> character;
                if(!strm.good()) throw std::runtime_error(errmsg);
                if(character != E(']')) throw std::runtime_error(errmsg);
                state = DONE;
                break;

            default:  // Interner Fehler, ungueltiger Status
                assert(!"MATLABMatrix(), invalid state");
                throw std::runtime_error(errmsg);
            }
        }
        while(state != DONE);

        // Ergebnismatrix mit der richtigen Groesse anlegen, danach alle
        // Matrixelemente aus "Zwischenspeicher" in Ergebnismatrix kopieren.
        Matrix result(dim);
        assert(dim * dim == elements.size());
        if(dim * dim != elements.size()) throw std::runtime_error(errmsg);

        // std::copy() gibt bei VC++ 2013 eine Fehlermeldung...
        Matrix::iterator res_it = result.begin(), res_end = result.end();
        std::vector<double>::const_iterator elem_it = elements.begin();
        while(res_it != res_end) *res_it++ = *elem_it++;
        return result;
    }

    // Quadratische MATLAB-Matrix aus std::string bzw. std::wstring einlesen
    Matrix MATLABMatrix(const std::string  &s) { return MATLABMatrix<>(s); }
    Matrix MATLABMatrix(const std::wstring &s) { return MATLABMatrix<>(s); }

};  // namespace HMMatrix

#endif  // MATRIX_HPP_Defined
